@@ -3,8 +3,6 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Copyright (c) 1999-2003 Apple Computer, Inc.  All Rights Reserved.
- * 
  * This file contains Original Code and/or Modifications of Original Code
  * as defined in and that are subject to the Apple Public Source License
  * Version 2.0 (the 'License'). You may not use this file except in
@@ -46,7 +44,7 @@ struct literal4_data {
     unsigned long nfiles;	/* number of files with this section */
     unsigned long nliterals;	/* total number of literals in the input files*/
 				/*  merged into this section  */
-#endif DEBUG
+#endif /* DEBUG */
 };
 
 /* the number of entries in the hash table */
@@ -95,4 +93,4 @@ __private_extern__ void print_literal4_data(
 __private_extern__ void literal4_data_stats(
     struct literal4_data *data,
     struct merged_section *ms);
-#endif DEBUG
+#endif /* DEBUG */
