@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.1 (the "License").  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON- INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -103,8 +102,9 @@ int argc,
 char **argv,
 char **envp)
 {
+    int i;
     struct cmd_flags cmd_flags;
-    unsigned long i, j, table_size;
+    unsigned long j, table_size;
     struct arch_flag *arch_flags;
     unsigned long narch_flags;
     enum bool all_archs;
@@ -229,8 +229,8 @@ char **envp)
 	if(cmd_flags.nfiles != 1)
 	    usage();
 
-	for(i = 0; i < cmd_flags.nfiles; i++)
-	    ofile_process(files[i], arch_flags, narch_flags, all_archs, TRUE,
+	for(j = 0; j < cmd_flags.nfiles; j++)
+	    ofile_process(files[j], arch_flags, narch_flags, all_archs, TRUE,
 			  TRUE, FALSE, checksyms, &cmd_flags);
 
 	if(errors == 0)
@@ -288,7 +288,8 @@ void *cookie)
 		check_dynamic_binary(ofile, arch_name, cmd_flags->detail,
 				     cmd_flags->verification);
 
-	if(ofile->mh->filetype == MH_DYLIB)
+	if(ofile->mh->filetype == MH_DYLIB ||
+	   ofile->mh->filetype == MH_DYLIB_STUB)
 	    check_dylib(ofile, arch_name, cmd_flags->detail,
 			cmd_flags->verification, &debug);
 
@@ -343,7 +344,7 @@ void *cookie)
 	    if(symbols[i].n_un.n_strx == 0)
 		symbols[i].n_un.n_name = "";
 	    else if(symbols[i].n_un.n_strx < 0 ||
-		    symbols[i].n_un.n_strx > st->strsize)
+		    (unsigned long)symbols[i].n_un.n_strx > st->strsize)
 		symbols[i].n_un.n_name = "bad string index";
 	    else
 		symbols[i].n_un.n_name = symbols[i].n_un.n_strx + strings;
@@ -410,6 +411,7 @@ void *cookie)
 	    ofile->mh->filetype == MH_FVMLIB))
 	    return;
 	if((ofile->mh->filetype == MH_DYLIB ||
+	    ofile->mh->filetype == MH_DYLIB_STUB ||
 	    ofile->mh->filetype == MH_FVMLIB) &&
 	    (nfun == 0 || debug == TRUE))
 	    return;
@@ -499,7 +501,8 @@ enum bool verification)
 	 * undefined references it should be prebound.
 	 */
 	if((ofile->mh->filetype == MH_EXECUTE ||
-	    ofile->mh->filetype == MH_DYLIB) &&
+	    ofile->mh->filetype == MH_DYLIB ||
+	    ofile->mh->filetype == MH_DYLIB_STUB) &&
 	   (ofile->mh->flags & MH_NOUNDEFS) == MH_NOUNDEFS){
 
 	    if((ofile->mh->flags & MH_PREBOUND) != MH_PREBOUND){
