@@ -3,21 +3,20 @@
  *
  * @APPLE_LICENSE_HEADER_START@
  * 
- * Portions Copyright (c) 1999 Apple Computer, Inc.  All Rights
- * Reserved.  This file contains Original Code and/or Modifications of
- * Original Code as defined in and that are subject to the Apple Public
- * Source License Version 1.1 (the "License").  You may not use this file
- * except in compliance with the License.  Please obtain a copy of the
- * License at http://www.apple.com/publicsource and read it before using
- * this file.
+ * This file contains Original Code and/or Modifications of Original Code
+ * as defined in and that are subject to the Apple Public Source License
+ * Version 2.0 (the 'License'). You may not use this file except in
+ * compliance with the License. Please obtain a copy of the License at
+ * http://www.opensource.apple.com/apsl/ and read it before using this
+ * file.
  * 
  * The Original Code and all software distributed under the License are
- * distributed on an "AS IS" basis, WITHOUT WARRANTY OF ANY KIND, EITHER
+ * distributed on an 'AS IS' basis, WITHOUT WARRANTY OF ANY KIND, EITHER
  * EXPRESS OR IMPLIED, AND APPLE HEREBY DISCLAIMS ALL SUCH WARRANTIES,
  * INCLUDING WITHOUT LIMITATION, ANY WARRANTIES OF MERCHANTABILITY,
- * FITNESS FOR A PARTICULAR PURPOSE OR NON- INFRINGEMENT.  Please see the
- * License for the specific language governing rights and limitations
- * under the License.
+ * FITNESS FOR A PARTICULAR PURPOSE, QUIET ENJOYMENT OR NON-INFRINGEMENT.
+ * Please see the License for the specific language governing rights and
+ * limitations under the License.
  * 
  * @APPLE_LICENSE_HEADER_END@
  */
@@ -57,14 +56,14 @@ static void open_target(
     char *target,
     struct ofile *ofile,
     struct nlist **symbols,
-    long *nsyms,
+    unsigned long *nsyms,
     char **strings,
-    long *strsize,
+    unsigned long *strsize,
     char **text,
-    long *textaddr,
-    long *textsize,
-    long *text_nsect,
-    long *data_nsect);
+    unsigned long *textaddr,
+    unsigned long *textsize,
+    unsigned long *text_nsect,
+    unsigned long *data_nsect);
 
 static void make_sorted_text_symbols(
     struct nlist *symbols,
@@ -80,11 +79,11 @@ static void make_global_symbols(
     long *nglobal_symbols);
 
 static long get_target_addr(
-    long value,
+    unsigned long value,
     char *name,
     char *text,
-    long addr,
-    long size,
+    unsigned long addr,
+    unsigned long size,
     char *target);
 
 static int cmp_qsort(
@@ -103,7 +102,7 @@ struct ext {
     char *name;		  /* symbol name */
     long type;		  /* symbol type (n_type & N_TYPE) */
     long sect;		  /* symbol section (n_sect) */
-    long value;		  /* symbol value */
+    unsigned long value;  /* symbol value */
     char *branch_target;  /* if non-zero the branch target symbol name */
     struct ext *next;	  /* next ext on the hash chain */
 };
@@ -524,16 +523,16 @@ open_target(
 char *target, 	       /* name of the target shared library to open */
 struct ofile *ofile,   /* pointer to the ofile struct to use for target */
 struct nlist **symbols,/* pointer to the symbol table to return */
-long *nsyms,	       /* pointer to the number of symbols to return */
+unsigned long *nsyms,  /* pointer to the number of symbols to return */
 char **strings,	       /* pointer to the string table to return */
-long *strsize,	       /* pointer to the string table size to return */
+unsigned long *strsize,/* pointer to the string table size to return */
 char **text,	       /* pointer to the text segment to return */
-long *textaddr,	       /* pointer to the text segment's address to return */
-long *textsize,	       /* pointer to the text segment's size to return */
-long *text_nsect,      /* pointer to the text section number to return */
-long *data_nsect)      /* pointer to the data section number to return */
+unsigned long *textaddr,/* pointer to the text segment's address to return */
+unsigned long *textsize,/* pointer to the text segment's size to return */
+unsigned long *text_nsect,/* pointer to the text section number to return */
+unsigned long *data_nsect)/* pointer to the data section number to return */
 {
-    long i, j, nsect;
+    unsigned long i, j, nsect;
     struct load_command *lcp;
     struct symtab_command *stp;
     struct segment_command *text_seg, *sgp;
@@ -688,7 +687,7 @@ long *data_nsect)      /* pointer to the data section number to return */
 	    if(((*symbols)[i].n_type & N_EXT) != N_EXT)
 		continue;
 	    if((*symbols)[i].n_un.n_strx < 0 ||
-	       (*symbols)[i].n_un.n_strx > *strsize)
+	       (unsigned long)((*symbols)[i].n_un.n_strx) > *strsize)
 		fatal("Bad string table index (%ld) for symbol %ld in: %s", 
 	    	      (*symbols)[i].n_un.n_strx, i, target);
 	    (*symbols)[i].n_un.n_name = *strings + (*symbols)[i].n_un.n_strx;
@@ -772,15 +771,14 @@ long *nglobal_symbols)
 static
 long
 get_target_addr(
-long value,
+unsigned long value,
 char *name,
 char *text,
-long addr,
-long size,
+unsigned long addr,
+unsigned long size,
 char *target)
 {
-    long offset;
-    unsigned long target_addr;
+    unsigned long offset, target_addr;
 
 	offset = value - addr;
 	if(offset < 0 || offset > size)
